<?php
namespace Ep\Middleware;

use Interop\Container\ContainerInterface;
use Laminas\ServiceManager\Exception\ServiceNotCreatedException;
use Laminas\ServiceManager\Factory\FactoryInterface;
use Laminas\Stratigility\Middleware\PathMiddlewareDecorator;
use Laminas\Stratigility\MiddlewarePipe;
use Mezzio\Middleware\LazyLoadingMiddleware;
use Mezzio\MiddlewareContainer;
use function Laminas\Stratigility\path;

/**
 * Stvara novu instancu EP Aplikacije.
 *
 * @package Ep\Middleware
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
final class AppFactory implements FactoryInterface
{
    /**
     * Stvara aplikaciju.
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @param  null|array $options
     *
     * @return object
     *
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        $config = $container->get('config')['app'] ?? [];
        $app = new MiddlewarePipe();
        $middlewares = array_unique($config['middleware'] ?? []);

        foreach ($middlewares as $key => $middleware) {
            if (!is_string($middleware)) {
                continue;
            }

            if (isset($config['middleware'][$middleware]) && false === $config['middleware'][$middleware]) {
                continue;
            }

            if (!$container->has($middleware)) {
                throw new ServiceNotCreatedException(sprintf(
                    "Middleware s imenom '%s' ne postoji.", $middleware
                ));
            }

            $middleware = new LazyLoadingMiddleware($container->get(MiddlewareContainer::class), $middleware);

            if (is_string($key)) {
                $middleware = new PathMiddlewareDecorator($key, $middleware);
            }

            $app->pipe($middleware);
        }

        return $app;
    }
}
